﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace PIOSample
{
    public partial class Form1 : Form
    {
        short DevId;
        Boolean ChangeLock = true;//このビットがtrueの時はビット出力設定しない
        public Form1()
        {
            InitializeComponent();
        }

        private void Control_Enable(Boolean state)
        {
            DevClose.Enabled = state;
            DevOpen.Enabled = !state;
            IDNUM.Enabled = !state;
            Dev1SetBtn.Enabled = state;
            Dev2SetBtn.Enabled = state;
            ChangeLock = !state;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // ***フォームの表示開始時の初期化処理***
            Control_Enable(false); //***
            IDNUM.SelectedIndex = 0; // '***初期選択IDを0にする
            Dev1PortASelIn.Checked = true;
            Dev1PortBSelIn.Checked = true;
            Dev1PortCLSelIn.Checked = true;
            Dev1PortCHSelIn.Checked = true;
            Dev2PortASelIn.Checked = true;
            Dev2PortBSelIn.Checked = true;
            Dev2PortCLSelIn.Checked = true;
            Dev2PortCHSelIn.Checked = true;
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            //***アプリケーション終了時の処理
            if( DevClose.Enabled )
            {
                TUSBPIO.Tusbpio_Device_Close(DevId);//***デバイスクローズ
            }
        }

        private void DevOpen_Click(object sender, EventArgs e)
        {
            short RetCode;

            //***デバイスのオープン***
            DevId = (short) IDNUM.SelectedIndex;
            RetCode = TUSBPIO.Tusbpio_Device_Open(DevId);
            if( RetCode == 0 )
            {
                Control_Enable(true); //各機能ボタンを有効にする
                Timer1.Interval = 300; //タイマーインターバル
                Timer1.Enabled = true; //タイマー開始
            }
            else
            {
                MessageBox.Show(TUSBPIO.GetErrMessage1(RetCode)); //***オープンに失敗した場合
            }
        }

        private void DevClose_Click(object sender, EventArgs e)
        {
            //***デバイスのクローズ***
            Timer1.Enabled = false; //タイマー停止
            TUSBPIO.Tusbpio_Device_Close(DevId); //***デバイスクローズ
            Control_Enable(false); //***ボタンの有効無効切り替え
        }

        private System.Drawing.Image GetLampImage(byte bitmap, byte bit)
        {
            byte BitMask;
            BitMask = (byte) Math.Pow(2,bit);
            return ( ( ( bitmap & BitMask ) == 0 ) ? ImageList1.Images[1] : ImageList1.Images[0]);
        }

        private void Timer1_Tick(object sender, EventArgs e)
        {
            short RetCode;
            byte[,] PortState = new byte[2,3];
            byte port;

            // Device 1からのデータの取得
            for( port = 0 ; port < 3 ;port++ )
            {
                RetCode = TUSBPIO.Tusbpio_Dev1_Read(DevId,port,out PortState[0,port]);
                if( RetCode != 0 )
                {
                    Timer1.Enabled = false;
                    MessageBox.Show(TUSBPIO.GetErrMessage2(RetCode));
                    return;
                }
            }
            // Device 2からのデータの取得
            for( port = 0 ; port < 3 ;port++ )
            {
                RetCode = TUSBPIO.Tusbpio_Dev2_Read(DevId,port,out PortState[1,port]);
                if( RetCode != 0 )
                {
                    Timer1.Enabled = false;
                    MessageBox.Show(TUSBPIO.GetErrMessage2(RetCode));
                    return;
                }
            }
            // Device1 PortA
            Dev1PortA_I0.Image = GetLampImage(PortState[0, 0], 0);
            Dev1PortA_I1.Image = GetLampImage(PortState[0, 0], 1);
            Dev1PortA_I2.Image = GetLampImage(PortState[0, 0], 2);
            Dev1PortA_I3.Image = GetLampImage(PortState[0, 0], 3);
            Dev1PortA_I4.Image = GetLampImage(PortState[0, 0], 4);
            Dev1PortA_I5.Image = GetLampImage(PortState[0, 0], 5);
            Dev1PortA_I6.Image = GetLampImage(PortState[0, 0], 6);
            Dev1PortA_I7.Image = GetLampImage(PortState[0, 0], 7);
            // Device1 PortB
            Dev1PortB_I0.Image = GetLampImage(PortState[0, 1], 0);
            Dev1PortB_I1.Image = GetLampImage(PortState[0, 1], 1);
            Dev1PortB_I2.Image = GetLampImage(PortState[0, 1], 2);
            Dev1PortB_I3.Image = GetLampImage(PortState[0, 1], 3);
            Dev1PortB_I4.Image = GetLampImage(PortState[0, 1], 4);
            Dev1PortB_I5.Image = GetLampImage(PortState[0, 1], 5);
            Dev1PortB_I6.Image = GetLampImage(PortState[0, 1], 6);
            Dev1PortB_I7.Image = GetLampImage(PortState[0, 1], 7);
            // Device1 PortC
            Dev1PortC_I0.Image = GetLampImage(PortState[0, 2], 0);
            Dev1PortC_I1.Image = GetLampImage(PortState[0, 2], 1);
            Dev1PortC_I2.Image = GetLampImage(PortState[0, 2], 2);
            Dev1PortC_I3.Image = GetLampImage(PortState[0, 2], 3);
            Dev1PortC_I4.Image = GetLampImage(PortState[0, 2], 4);
            Dev1PortC_I5.Image = GetLampImage(PortState[0, 2], 5);
            Dev1PortC_I6.Image = GetLampImage(PortState[0, 2], 6);
            Dev1PortC_I7.Image = GetLampImage(PortState[0, 2], 7);
            // Device2 PortA
            Dev2PortA_I0.Image = GetLampImage(PortState[1, 0], 0);
            Dev2PortA_I1.Image = GetLampImage(PortState[1, 0], 1);
            Dev2PortA_I2.Image = GetLampImage(PortState[1, 0], 2);
            Dev2PortA_I3.Image = GetLampImage(PortState[1, 0], 3);
            Dev2PortA_I4.Image = GetLampImage(PortState[1, 0], 4);
            Dev2PortA_I5.Image = GetLampImage(PortState[1, 0], 5);
            Dev2PortA_I6.Image = GetLampImage(PortState[1, 0], 6);
            Dev2PortA_I7.Image = GetLampImage(PortState[1, 0], 7);
            // Device2 PortB
            Dev2PortB_I0.Image = GetLampImage(PortState[1, 1], 0);
            Dev2PortB_I1.Image = GetLampImage(PortState[1, 1], 1);
            Dev2PortB_I2.Image = GetLampImage(PortState[1, 1], 2);
            Dev2PortB_I3.Image = GetLampImage(PortState[1, 1], 3);
            Dev2PortB_I4.Image = GetLampImage(PortState[1, 1], 4);
            Dev2PortB_I5.Image = GetLampImage(PortState[1, 1], 5);
            Dev2PortB_I6.Image = GetLampImage(PortState[1, 1], 6);
            Dev2PortB_I7.Image = GetLampImage(PortState[1, 1], 7);
            // Device2 PortC
            Dev2PortC_I0.Image = GetLampImage(PortState[1, 2], 0);
            Dev2PortC_I1.Image = GetLampImage(PortState[1, 2], 1);
            Dev2PortC_I2.Image = GetLampImage(PortState[1, 2], 2);
            Dev2PortC_I3.Image = GetLampImage(PortState[1, 2], 3);
            Dev2PortC_I4.Image = GetLampImage(PortState[1, 2], 4);
            Dev2PortC_I5.Image = GetLampImage(PortState[1, 2], 5);
            Dev2PortC_I6.Image = GetLampImage(PortState[1, 2], 6);
            Dev2PortC_I7.Image = GetLampImage(PortState[1, 2], 7);
        }

        private void Dev1SetBtn_Click(object sender, EventArgs e)
        {
            short RetCode;
            byte ModeBitmap;

            ModeBitmap = 0x80; //モード設定 mode 0

            //Dev1 PortA 入出力方向
            ModeBitmap |=  (byte) (Dev1PortASelIn.Checked ? 0x10 : 0); //第4ビットを1

            //Dev1 PortB 入出力方向
            ModeBitmap |=  (byte) (Dev1PortBSelIn.Checked ? 0x2 : 0); //第1ビットを1

            //Dev1 PortC 下位　入出力方向
            ModeBitmap |=  (byte) (Dev1PortCLSelIn.Checked ? 0x1 : 0); //第0ビットを1

            //dev1 PortC 上位　入出力方向
            ModeBitmap |=  (byte) (Dev1PortCHSelIn.Checked ? 0x8 : 0); //第3ビットを1

            RetCode = TUSBPIO.Tusbpio_Dev1_Write(DevId, 3, ModeBitmap);
            if( RetCode != 0 )
                MessageBox.Show(TUSBPIO.GetErrMessage2(RetCode));

            //モードを設定すると出力ビットパターンが初期化される
            ChangeLock = true;
            Dev1PortA_O0.Checked = false;
            Dev1PortA_O1.Checked = false;
            Dev1PortA_O2.Checked = false;
            Dev1PortA_O3.Checked = false;
            Dev1PortA_O4.Checked = false;
            Dev1PortA_O5.Checked = false;
            Dev1PortA_O6.Checked = false;
            Dev1PortA_O7.Checked = false;
            Dev1PortB_O0.Checked = false;
            Dev1PortB_O1.Checked = false;
            Dev1PortB_O2.Checked = false;
            Dev1PortB_O3.Checked = false;
            Dev1PortB_O4.Checked = false;
            Dev1PortB_O5.Checked = false;
            Dev1PortB_O6.Checked = false;
            Dev1PortB_O7.Checked = false;
            Dev1PortC_O0.Checked = false;
            Dev1PortC_O1.Checked = false;
            Dev1PortC_O2.Checked = false;
            Dev1PortC_O3.Checked = false;
            Dev1PortC_O4.Checked = false;
            Dev1PortC_O5.Checked = false;
            Dev1PortC_O6.Checked = false;
            Dev1PortC_O7.Checked = false;
            ChangeLock = false;
        }

        private void Dev2SetBtn_Click(object sender, EventArgs e)
        {
            short RetCode;
            byte ModeBitmap;

            ModeBitmap = 0x80; //モード設定 mode 0

            //Dev2 PortA 入出力方向
            ModeBitmap |= (byte)(Dev2PortASelIn.Checked ? 0x10 : 0); //第4ビットを1

            //Dev2 PortB 入出力方向
            ModeBitmap |= (byte)(Dev2PortBSelIn.Checked ? 0x2 : 0); //第1ビットを1

            //Dev2 PortC 下位　入出力方向
            ModeBitmap |= (byte)(Dev2PortCLSelIn.Checked ? 0x1 : 0); //第0ビットを1

            //dev1 PortC 上位　入出力方向
            ModeBitmap |= (byte)(Dev2PortCHSelIn.Checked ? 0x8 : 0); //第3ビットを1

            RetCode = TUSBPIO.Tusbpio_Dev2_Write(DevId, 3, ModeBitmap);
            if (RetCode != 0)
                MessageBox.Show(TUSBPIO.GetErrMessage2(RetCode));

            //モードを設定すると出力ビットパターンが初期化される
            ChangeLock = true;
            Dev2PortA_O0.Checked = false;
            Dev2PortA_O1.Checked = false;
            Dev2PortA_O2.Checked = false;
            Dev2PortA_O3.Checked = false;
            Dev2PortA_O4.Checked = false;
            Dev2PortA_O5.Checked = false;
            Dev2PortA_O6.Checked = false;
            Dev2PortA_O7.Checked = false;
            Dev2PortB_O0.Checked = false;
            Dev2PortB_O1.Checked = false;
            Dev2PortB_O2.Checked = false;
            Dev2PortB_O3.Checked = false;
            Dev2PortB_O4.Checked = false;
            Dev2PortB_O5.Checked = false;
            Dev2PortB_O6.Checked = false;
            Dev2PortB_O7.Checked = false;
            Dev2PortC_O0.Checked = false;
            Dev2PortC_O1.Checked = false;
            Dev2PortC_O2.Checked = false;
            Dev2PortC_O3.Checked = false;
            Dev2PortC_O4.Checked = false;
            Dev2PortC_O5.Checked = false;
            Dev2PortC_O6.Checked = false;
            Dev2PortC_O7.Checked = false;
            ChangeLock = false;
        }

        private void Dev1PortA_O_Changed(object sender, EventArgs e)
        {
            short RetCode;
            byte Bitmap = 0;

            if( !ChangeLock)
            {
                Bitmap |= (byte) ( Dev1PortA_O0.Checked ? 0x1 : 0 );
                Bitmap |= (byte) ( Dev1PortA_O1.Checked ? 0x2 : 0 );
                Bitmap |= (byte) ( Dev1PortA_O2.Checked ? 0x4 : 0 );
                Bitmap |= (byte) ( Dev1PortA_O3.Checked ? 0x8 : 0 );
                Bitmap |= (byte) ( Dev1PortA_O4.Checked ? 0x10 : 0 );
                Bitmap |= (byte) ( Dev1PortA_O5.Checked ? 0x20 : 0 );
                Bitmap |= (byte) ( Dev1PortA_O6.Checked ? 0x40 : 0 );
                Bitmap |= (byte) ( Dev1PortA_O7.Checked ? 0x80 : 0 );

                RetCode = TUSBPIO.Tusbpio_Dev1_Write(DevId,0,Bitmap);
                if( RetCode != 0 )
                {
                    MessageBox.Show(TUSBPIO.GetErrMessage2(RetCode));
                }
            }
        }

        private void Dev1PortB_O_Changed(object sender, EventArgs e)
        {
            short RetCode;
            byte Bitmap = 0;

            if (!ChangeLock)
            {
                Bitmap |= (byte)(Dev1PortB_O0.Checked ? 0x1 : 0);
                Bitmap |= (byte)(Dev1PortB_O1.Checked ? 0x2 : 0);
                Bitmap |= (byte)(Dev1PortB_O2.Checked ? 0x4 : 0);
                Bitmap |= (byte)(Dev1PortB_O3.Checked ? 0x8 : 0);
                Bitmap |= (byte)(Dev1PortB_O4.Checked ? 0x10 : 0);
                Bitmap |= (byte)(Dev1PortB_O5.Checked ? 0x20 : 0);
                Bitmap |= (byte)(Dev1PortB_O6.Checked ? 0x40 : 0);
                Bitmap |= (byte)(Dev1PortB_O7.Checked ? 0x80 : 0);

                RetCode = TUSBPIO.Tusbpio_Dev1_Write(DevId, 1, Bitmap);
                if (RetCode != 0)
                {
                    MessageBox.Show(TUSBPIO.GetErrMessage2(RetCode));
                }
            }
        }

        private void Dev1PortC_O_Changed(object sender, EventArgs e)
        {
            short RetCode;
            byte Bitmap = 0;

            if (!ChangeLock)
            {
                Bitmap |= (byte)(Dev1PortC_O0.Checked ? 0x1 : 0);
                Bitmap |= (byte)(Dev1PortC_O1.Checked ? 0x2 : 0);
                Bitmap |= (byte)(Dev1PortC_O2.Checked ? 0x4 : 0);
                Bitmap |= (byte)(Dev1PortC_O3.Checked ? 0x8 : 0);
                Bitmap |= (byte)(Dev1PortC_O4.Checked ? 0x10 : 0);
                Bitmap |= (byte)(Dev1PortC_O5.Checked ? 0x20 : 0);
                Bitmap |= (byte)(Dev1PortC_O6.Checked ? 0x40 : 0);
                Bitmap |= (byte)(Dev1PortC_O7.Checked ? 0x80 : 0);

                RetCode = TUSBPIO.Tusbpio_Dev1_Write(DevId, 2, Bitmap);
                if (RetCode != 0)
                {
                    MessageBox.Show(TUSBPIO.GetErrMessage2(RetCode));
                }
            }
        }

        private void Dev2PortA_O_Changed(object sender, EventArgs e)
        {
            short RetCode;
            byte Bitmap = 0;

            if (!ChangeLock)
            {
                Bitmap |= (byte)(Dev2PortA_O0.Checked ? 0x1 : 0);
                Bitmap |= (byte)(Dev2PortA_O1.Checked ? 0x2 : 0);
                Bitmap |= (byte)(Dev2PortA_O2.Checked ? 0x4 : 0);
                Bitmap |= (byte)(Dev2PortA_O3.Checked ? 0x8 : 0);
                Bitmap |= (byte)(Dev2PortA_O4.Checked ? 0x10 : 0);
                Bitmap |= (byte)(Dev2PortA_O5.Checked ? 0x20 : 0);
                Bitmap |= (byte)(Dev2PortA_O6.Checked ? 0x40 : 0);
                Bitmap |= (byte)(Dev2PortA_O7.Checked ? 0x80 : 0);

                RetCode = TUSBPIO.Tusbpio_Dev2_Write(DevId, 0, Bitmap);
                if (RetCode != 0)
                {
                    MessageBox.Show(TUSBPIO.GetErrMessage2(RetCode));
                }
            }
        }

        private void Dev2PortB_O_Changed(object sender, EventArgs e)
        {
            short RetCode;
            byte Bitmap = 0;

            if (!ChangeLock)
            {
                Bitmap |= (byte)(Dev2PortB_O0.Checked ? 0x1 : 0);
                Bitmap |= (byte)(Dev2PortB_O1.Checked ? 0x2 : 0);
                Bitmap |= (byte)(Dev2PortB_O2.Checked ? 0x4 : 0);
                Bitmap |= (byte)(Dev2PortB_O3.Checked ? 0x8 : 0);
                Bitmap |= (byte)(Dev2PortB_O4.Checked ? 0x10 : 0);
                Bitmap |= (byte)(Dev2PortB_O5.Checked ? 0x20 : 0);
                Bitmap |= (byte)(Dev2PortB_O6.Checked ? 0x40 : 0);
                Bitmap |= (byte)(Dev2PortB_O7.Checked ? 0x80 : 0);

                RetCode = TUSBPIO.Tusbpio_Dev2_Write(DevId, 1, Bitmap);
                if (RetCode != 0)
                {
                    MessageBox.Show(TUSBPIO.GetErrMessage2(RetCode));
                }
            }
        }

        private void Dev2PortC_O_Changed(object sender, EventArgs e)
        {
            short RetCode;
            byte Bitmap = 0;

            if (!ChangeLock)
            {
                Bitmap |= (byte)(Dev2PortC_O0.Checked ? 0x1 : 0);
                Bitmap |= (byte)(Dev2PortC_O1.Checked ? 0x2 : 0);
                Bitmap |= (byte)(Dev2PortC_O2.Checked ? 0x4 : 0);
                Bitmap |= (byte)(Dev2PortC_O3.Checked ? 0x8 : 0);
                Bitmap |= (byte)(Dev2PortC_O4.Checked ? 0x10 : 0);
                Bitmap |= (byte)(Dev2PortC_O5.Checked ? 0x20 : 0);
                Bitmap |= (byte)(Dev2PortC_O6.Checked ? 0x40 : 0);
                Bitmap |= (byte)(Dev2PortC_O7.Checked ? 0x80 : 0);

                RetCode = TUSBPIO.Tusbpio_Dev2_Write(DevId, 2, Bitmap);
                if (RetCode != 0)
                {
                    MessageBox.Show(TUSBPIO.GetErrMessage2(RetCode));
                }
            }
        }
    }
}
